<?php

namespace Drupal\cafe_reservation\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Database;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 *
 */
class ReservationAdminController extends ControllerBase {

  /**
   * The mail manager service.
   *
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  protected $mailManager;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Constructs a ReservationAdminController object.
   *
   * @param \Drupal\Core\Mail\MailManagerInterface $mail_manager
   *   The mail manager service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   */
  public function __construct(MailManagerInterface $mail_manager, MessengerInterface $messenger) {
    $this->mailManager = $mail_manager;
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.mail'),
      $container->get('messenger')
    );
  }

  /**
   * Display all reservations and provide status update options.
   */
  public function adminPage() {
    // Fetch all reservations from the database.
    $query = Database::getConnection()->select('cafe_reservations', 'r')
      ->fields('r', ['id', 'name', 'contact', 'email', 'reservation_date', 'table_size', 'status'])
      ->orderBy('reservation_date', 'ASC');
    $reservations = $query->execute()->fetchAll();

    // Build table headers.
    $header = [
      $this->t('ID'),
      $this->t('Name'),
      $this->t('Contact'),
      $this->t('Email'),
      $this->t('Reservation Date'),
      $this->t('Table Size'),
      $this->t('Status'),
      $this->t('Operations'),
    ];

    // Build rows.
    $rows = [];
    foreach ($reservations as $reservation) {
      $rows[] = [
        'id' => $reservation->id,
        'name' => $reservation->name,
        'contact' => $reservation->contact,
        'email' => $reservation->email,
        'reservation_date' => date('Y-m-d H:i', $reservation->reservation_date),
        'table_size' => $reservation->table_size,
        'status' => $reservation->status,
        'operations' => [
          'data' => $this->buildOperations($reservation->id, $reservation->status),
        ],
      ];
    }

    // Render table.
    return [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => $this->t('No reservations found.'),
    ];
  }

  /**
   * Build operation links for each reservation.
   */
  private function buildOperations($id, $status) {
    $links = [];

    if ($status !== 'approved') {
      $url = Url::fromRoute('cafe_reservation.status_update', ['action' => 'approve', 'id' => $id]);
      $links['approve'] = [
        'title' => $this->t('Approve'),
        'url' => $url,
      ];
    }

    if ($status !== 'cancelled') {
      $url = Url::fromRoute('cafe_reservation.status_update', ['action' => 'cancel', 'id' => $id]);
      $links['cancel'] = [
        'title' => $this->t('Cancel'),
        'url' => $url,
      ];
    }

    return [
      '#type' => 'operations',
      '#links' => $links,
    ];
  }

  /**
   * Status update handler.
   */
  public function statusUpdate($action, $id) {
    if (in_array($action, ['approve', 'cancel'])) {
      $status = ($action === 'approve') ? 'approved' : 'cancelled';

      // Fetch reservation details.
      $reservation = Database::getConnection()->select('cafe_reservations', 'r')
        ->fields('r', ['name', 'email'])
        ->condition('id', $id)
        ->execute()
        ->fetchObject();

      if ($reservation) {
        // Update status in the database.
        Database::getConnection()->update('cafe_reservations')
          ->fields(['status' => $status])
          ->condition('id', $id)
          ->execute();

        // Send notification email.
        $this->sendStatusEmail($reservation->name, $reservation->email, $status);

        // Show confirmation message.
        $this->messenger->addMessage($this->t('Reservation has been @action.', ['@action' => $status]));
      }
    }

    return new RedirectResponse('/admin/reservations');
  }

  /**
   * Send email notification to the user about reservation status update.
   *
   * @param string $name
   *   The user's name.
   * @param string $email
   *   The user's email address.
   * @param string $status
   *   The new status of the reservation.
   */
  private function sendStatusEmail($name, $email, $status) {
    $module = 'cafe_reservation';
    $key = 'reservation_status_update';
    $to = $email;
    $params['message'] = $this->t('Hello @name, your reservation has been @status.', [
      '@name' => $name,
      '@status' => ucfirst($status),
    ]);
    $params['subject'] = $this->t('Your Reservation Status: @status', ['@status' => ucfirst($status)]);
    $langcode = $this->currentUser()->getPreferredLangcode();

    $send = TRUE;

    $result = $this->mailManager->mail($module, $key, $to, $langcode, $params, NULL, $send);

    if ($result['result'] !== TRUE) {
      $this->messenger->addMessage($this->t('There was a problem sending your email and it was not sent.'), 'error');
    }
    else {
      $this->messenger->addMessage($this->t('An email has been sent to the user notifying them of the status change.'));
    }
  }

}
