<?php

namespace Drupal\cafe_reservation\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * {@inheritdoc}
 */
class ReservationForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'cafe_reservation_form';
  }

  /**
   * The mail manager service.
   *
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  protected $mailManager;

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;
  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;
  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;
  /**
   * Database connection.
   */
  protected Connection $database;
  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected LanguageManagerInterface $languageManager;

  /**
   * Constructs a new StatusUpdateForm.
   *
   * @param \Drupal\Core\Mail\MailManagerInterface $mail_manager
   *   The mail manager service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Database\Connection $database
   *   Database connection.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   */
  public function __construct(MailManagerInterface $mail_manager, ConfigFactoryInterface $config_factory, AccountInterface $current_user, EntityTypeManagerInterface $entity_type_manager, Connection $database, LanguageManagerInterface $language_manager) {
    $this->mailManager = $mail_manager;
    $this->configFactory = $config_factory;
    $this->currentUser = $current_user;
    $this->entityTypeManager = $entity_type_manager;
    $this->database = $database;
    $this->languageManager = $language_manager;

  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.mail'),
      $container->get('config.factory'),
      $container->get('current_user'),
      $container->get('entity_type.manager'),
      $container->get('database'),
      $container->get('language_manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $user = $this->currentUser();
    $user_name = $user->getDisplayName();
    $user_email = $user->getEmail();

    $form['name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Your Name'),
      '#required' => TRUE,
      '#default_value' => $user_name,
    ];

    $form['contact'] = [
      '#type' => 'tel',
      '#title' => $this->t('Contact Number'),
      '#required' => TRUE,
      '#attributes' => ['pattern' => '[0-9]{10}'],
      '#description' => $this->t('Enter a valid 10-digit phone number.'),
    ];

    $form['email'] = [
      '#type' => 'email',
      '#title' => $this->t('Email Address'),
      '#required' => TRUE,
      '#default_value' => $user_email,
    ];

    // Add field for reservation date.
    $form['reservation_date'] = [
      '#type' => 'datetime',
      '#title' => $this->t('Reservation Date'),
      '#required' => TRUE,
    ];

    // Add field for table size.
    $form['table_size'] = [
      '#type' => 'select',
      '#title' => $this->t('Table Size'),
      '#options' => [1 => '1 Pax', 2 => '2 Pax', 4 => '4 Pax', 6 => '6 Pax', 8 => '8 Pax', 10 => '10 Pax'],
      '#required' => TRUE,
    ];

    // Add submit button.
    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Reserve Table'),
    ];

    return $form;
  }

  /**
   * Form validation handler.
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // Validate contact number.
    if (!preg_match('/^[0-9]{10}$/', $form_state->getValue('contact'))) {
      $form_state->setErrorByName('contact', $this->t('The contact number must be 10 digits.'));
    }

    // Validate email format.
    if (!filter_var($form_state->getValue('email'), FILTER_VALIDATE_EMAIL)) {
      $form_state->setErrorByName('email', $this->t('The email address is not valid.'));
    }
  }

  /**
   * Form submission handler.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Insert the reservation details into the database.
    $reservation_id = $this->database->insert('cafe_reservations')
      ->fields([
        'user_id' => $this->currentUser->id(),
        'name' => $form_state->getValue('name'),
        'contact' => $form_state->getValue('contact'),
        'email' => $form_state->getValue('email'),
        'reservation_date' => $form_state->getValue('reservation_date')->getTimestamp(),
        'table_size' => $form_state->getValue('table_size'),
        'status' => 'pending',
      ])->execute();

    // Notify the user and the admin.
    $this->sendNotificationEmails($form_state);

    $this->messenger()->addMessage($this->t('Your reservation has been submitted.'));
  }

  /**
   * Sends email notifications to the admin and user.
   */
  protected function sendNotificationEmails(FormStateInterface $form_state) {
    $mailManager = $this->mailManager;

    // Prepare user email details.
    $user_email = $form_state->getValue('email');
    $params_user = [
      'subject' => $this->t('Your Reservation at the Cafe'),
      'message' => $this->t('Thank you for your reservation on @date. We will confirm shortly.', [
        '@date' => $form_state->getValue('reservation_date')->format('Y-m-d H:i'),
      ]),
    ];

    // Send email to the user.
    $mailManager->mail('cafe_reservation', 'reservation_confirmation', $user_email, $this->languageManager->getDefaultLanguage()->getId(), $params_user);

    // Prepare admin email details.
    $admin_email = $this->configFactory->get('system.site')->get('mail');
    $params_admin = [
      'subject' => $this->t('New Reservation Received'),
      'message' => $this->t('A new reservation has been placed by @name for @date. Please review.', [
        '@name' => $form_state->getValue('name'),
        '@date' => $form_state->getValue('reservation_date')->format('Y-m-d H:i'),
      ]),
    ];

    // Send email to the admin.
    $mailManager->mail('cafe_reservation', 'reservation_notification', $admin_email, $this->languageManager->getDefaultLanguage()->getId(), $params_admin);
  }

}
